#!/usr/bin/perl
#-----------------------------------------------------------------------
# Licensed Materials - Property of IBM
#
# IBM Tape System Library Manager
#
# (C) Copyright IBM Corp. 2004, 2013 All Rights Reserved
#
# US Government Users Restricted Rights - Use,  duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#-----------------------------------------------------------------------
#
# @version 1.11 - 14/03/17 21:22:31
#
# @brief Tool to collect log files and informations relevant for debugging
#
#-----------------------------------------------------------------------

package IBM_ermmCollectLogs;
use strict;

# This tool (ermmCollectLogs) uses the TSLM CLI tool (ermmtool) to
# collect information from the Media Manager. If a password is set
# on the target Media Manager for ermmtool access, edit the variable
# $ermmtool_password below to set the password string so that this
# tool will execute the ermmtool command with the password.
# Or, set a null string ("") to execute the ermmtool without the
# password option (which is the default setting for the variable.
#
# Note: You can also set a default password for executing the ermmtool
# command by using the ermmtool configuration file (ermmcli.conf),
# instead. For details about the ermmtool configuration file, see 
# the TSLM User's Guide.

my $ermmtool_password = ""; 

#============ Don't change anything below this line ====================
#=========== unless you really know what you're doing ==================


#
# store common part proprietary statement in memory
#
my $copyright = << 'ENDCOPYRIGHT';
IBM Tape System Library Manager
Licensed Materials - Property of IBM
(C) Copyright IBM Corp. 2004, 2013 All Rights Reserved
ENDCOPYRIGHT

#
# store US Government Users Restricted Rights Legend in memory
#
my $usgovrestricted = << 'ENDUSGOVRESTRICTED';
US Government Users Restricted Rights - Use, duplication or
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
ENDUSGOVRESTRICTED

#######################################################
#
# Setup external modules
#

use Getopt::Long;
use File::Copy;
use File::Find;
use File::Path;
use Config;

use constant REGISTRY => do{ eval{ require Win32API::Registry }};
use constant TAR => do{ eval{ require Archive::Tar }};
use constant Zip => do{ eval{ require Archive::Zip }};
use constant DOS => ($Config{'osname'} =~ /win|dos/i);
use constant READKEY => do{ eval{ require Term::ReadKey }};
use constant S_IRWXU => oct("700");

my $pathsep    = DOS ? '\\' : '/';
my $pathsepreg = DOS ? '\\\\' : '/';

#######################################################
#
# Usage and option parsing
#

#
# about
#
my $about = << 'ENDABOUT';
Tool to collect log files and informations relevant for debugging
ENDABOUT

#
# usage
#
my $usage = << 'ENDUSAGE';
Syntax: scratch.pl [options]
Available options:
  --help|-h        Print this help
  --out|-o         File where to output the log
  --tmp|-t         Useable temporary directory
  --base|-b        Base directory of TSLM
  --quiet|-q       No output
  --debug|-d       Increase debug level
ENDUSAGE

my %info=();                      # used to store collected information

my $basename = "ermmCollectLogs"; # name used for log package
my $haswarnings = 0;

my %options=();
$options{'temp'}       = searchTmp();
$options{'out'}        = "";

GetOptions("help|h"    => sub {print $copyright.$about.$usage; exit },
           "out|o=s"   => \$options{'out'},
           "tmp|t=s"   => \$options{'temp'},
           "base|b=s"  => \$options{'base'},
           "debug|d+"  => \$options{'debug'},
           "quiet|q"   => \$options{'quiet'});

$options{'temp'} .= "${pathsep}" if $options{'temp'} =~ /[^\/\\]$/;

printlog(1, $copyright.$about."\n");

#######################################################
#
# scratch
#


#######################################################
#
# basic message functions
#

#
# print warning message
#
sub warning {
    $haswarnings++;
    print "warning: ".join '',@_ ;
    print LOG @_;
}

#
# print an activity log message
#
sub printlog {
    my $level = shift;
    print @_ if !$options{'quiet'} && $level <= 1;
    print LOG @_;
}

#
# print a system info message
#
sub printsysinfo {
    print SYSINFO @_;
}

#
# print debug message
#
sub debug {
    my $level = shift;
    if (@_[0] =~ /^\n/) {
        @_[0] =~ s/^\n//m;
        print "\n" if $options{'debug'} >= $level;
        print LOG "\n" if $options{'debug'} >= $level;
    }
    print "debug: ".(join '',@_) if $options{'debug'} >= $level;
    print LOG "debug: ".(join '',@_) if $options{'debug'} >= $level;
}

#
# print animation char on screen.
#
sub animation {
    my @animchar = qw(| / - \\);
    my $pos = time % 4;
    print "$animchar[$pos]\b";
}


#######################################################
#
# collect informations on a specific topic
#


#######################################################
#
# locate TSLM directories
#

#
# Try common locations of temporary directories
#
sub searchTmp {
    $options{'temp'} = "/tmp";
    return $options{'temp'} if (-d $options{'temp'});
    $options{'temp'} = "C:\\temp";
    return $options{'temp'} if (-d $options{'temp'});
    $options{'temp'} = "C:\\windows\\temp";
    return $options{'temp'} if (-d $options{'temp'});
    $options{'temp'} = "~/tmp";
    return $options{'temp'} if (-d $options{'temp'});
    debug 1, "Couldn't autodetect tmp path.\n";
}

#
# try to find TSLM
#
sub collect_ermmlocation {
    printlog 2, "Locate TSLM\n";
    $info{'TSLM path'} = '';

    if (defined $options{'base'} && $options{'base'} ne "") {
        if (! (-d "$options{'base'}")) {
            warning "The TSLM base directory doesn't exist.\n";
            return;
        }
        $info{'TSLM path'} = $options{'base'};
        $info{'TSLM path'} .= "${pathsep}" if $info{'TSLM path'} =~ /[^\/\\]$/;
    } elsif (!try_ermmlocation()) {
        warning "Couldn't find TSLM path.\n";
        return;
    }

    printsysinfo "TSLM path: $info{'TSLM path'}\n";

    if (! -d "$info{'TSLM path'}conf") {
        warning "config path $info{'TSLM path'}conf does not exist.\n";
        $info{'TSLM conf'} = $info{'TSLM path'};
        return;
    }
    $info{'TSLM conf'} = "$info{'TSLM path'}conf${pathsep}";

    if (! (-f "$info{'TSLM path'}bin${pathsep}MediaManager" ||
           -f "$info{'TSLM path'}bin${pathsep}LibraryManager" ||
           -f "$info{'TSLM path'}bin${pathsep}ermmGenHDM" )) {
        warning "$info{'TSLM path'}bin${pathsep} doesn't contain either of\nMediaManager, LibraryManager or ermmGenHDM.\n";
        return;
    }
    printlog 1, "Located TSLM in $info{'TSLM path'}\n";
}

#
# check if TSLM is in one of the default places
#
sub try_ermmlocation {
    my @ermmlocations = ('C:\\Programme\\IBM\\ermm\\', '/opt/IBM/ermm/', 'C:\\Program Files\\IBM\\ermm\\', 'C:\\Program Files (x86)\\IBM\\ermm\\', './');
    my @ermmfiles     = ("bin${pathsep}MediaManager", "bin${pathsep}LibraryManager", "bin${pathsep}ermmGenHDM",
                         "conf${pathsep}genhdm.cfg", "conf${pathsep}mm.xml");
    foreach my $ermmlocation (@ermmlocations) {
        debug 1, "Test TSLM location $ermmlocation\n";
        foreach my $ermmfile (@ermmfiles) {
            if (-f "${ermmlocation}${ermmfile}") {
                debug 1, "Found TSLM file: $ermmlocation$ermmfile\n";
                $info{'TSLM path'} = $ermmlocation;
                return 1;
            }
        }
    }
    return 0;
}


#######################################################
#
# copy log and config files
#

#
# Copy TSLM Config Files from default locations
#
sub collect_conffiles {
    printlog 1, "Copy conf files into temporary directory\n";

    printsysinfo "TSLM config path: $info{'TSLM conf'}\n";

    if (!copy_files( $info{'TSLM conf'}, "$basename${pathsep}conf${pathsep}", '^(lm-.*\.xml|dbuserexit\.conf|eRMM\.baroc|genhdm\.cfg|hdmdrive\.cfg|mm\.xml|tecadapt\.conf|LibBalConf\.xml|physicaldrive\.conf)$' )) {
        warning "No config files found in $info{'TSLM conf'}\n";
    }

    if (copy_files( "$info{'TSLM path'}client${pathsep}tsm${pathsep}", "$basename${pathsep}conf${pathsep}", '^(.*\.cfg)$' )) {
        printlog 2, "Copied client/tsm/*.cfg to conf\n";
    };
}

#
# Copy TSLM Log Files from default locations
#
sub collect_logfiles {
    printlog 1, "Copy log files from default location\n";
    my $dst;
    if (-d "$info{'TSLM path'}log") {
        $info{'TSLM logs'} = "$info{'TSLM path'}log${pathsep}";
        $dst = "$basename${pathsep}logs${pathsep}";
        mkdir $dst, S_IRWXU;
    } else {
        printlog 2, "log path $info{'TSLM path'}log does not exist.\n";
        $info{'TSLM logs'} = "$info{'TSLM path'}";
        $dst = "$basename${pathsep}";
    }
    printsysinfo "TSLM log path: $info{'TSLM logs'}\n";

    if (!copy_files( $info{'TSLM logs'}, $dst, 
	   '^(lm-.*\.\d+\.log|ermmtrace\.\d+\.log|ermmGenHDM\.\d+\.log|ermmdb2install.log|ermmdbstart.log|ermmdb2install.err|ermmstart.log|Balance.txt|physicaldrive\.log)$' )) {
       warning "No log files found in $info{'TSLM logs'}\n";
    };

    &copy_scslogfiles("${pathsep}var${pathsep}adm${pathsep}ras", "${dst}SCS");
    &copy_scslogfiles("${pathsep}var${pathsep}log", "${dst}SCS");
}

#
# Copy SCS Log Files
#
sub copy_scslogfiles {
    my @scslogfiles = glob("$_[0]${pathsep}ibmscs.*");
    if (scalar(@scslogfiles) > 0) {
        my $scslogdst = "$_[1]$_[0]";
        printlog 1, "Copy SCS log files from $_[0] to $scslogdst\n";
        mkpath($scslogdst, {verbose => 0}, S_IRWXU);
        copy_files( "$_[0]${pathsep}", "$scslogdst${pathsep}", '^(ibmscs)');
    }
}

#
# Copy core files from default locations
#
sub collect_corefiles {
    printlog 1, "Copy core files from default location\n";

    # locate mm cores
    my @cores;
    return if !-d "$info{'TSLM path'}run";
    if (-d "$info{'TSLM path'}run${pathsep}mm") {
        push @cores, map "$info{'TSLM path'}run${pathsep}mm${pathsep}$_", get_direntries("$info{'TSLM path'}run${pathsep}mm${pathsep}",'^\d+-\d+-\d+-\d+:\d+:\d+$');
    }

    # locate lm cores
    my @lm = map "$info{'TSLM path'}run${pathsep}$_", get_direntries("$info{'TSLM path'}run${pathsep}",'^lm-.*$');
    debug 1, "found lm directories: @lm\n";
    foreach (@lm) {
        next if !-d "$_";
        push @cores, get_direntries("$_${pathsep}",'^\d+-\d+-\d+-\d+:\d+:\d+$');
     }
    debug 1, "found core dumps: @cores\n";

    # search for recent cores. recent = 3 days;
    my @recentcores = ();
    foreach (@cores) {
        #my ($year, $month, $day, $hour, $min, $sec) =
        #  $_ =~ /(\d+)-(\d+)-(\d+)-(\d+):(\d+):(\d+)$/;
        my $mtime = (stat($_))[9];
        if ($mtime + 3*24*60*60 > time) {
            push @recentcores, $_;
        }
    }
    debug 1, "recent cores: @recentcores\n";

    return if @recentcores == 0;

    # copy recent cores
    my $corepath = "$basename${pathsep}core${pathsep}";
    mkdir $corepath, S_IRWXU;
    foreach my $srccore (@recentcores) {
        my ($corefile) = $srccore =~ /^.*?([^${pathsepreg}]*)$/;

        copy_files( $srccore.${pathsepreg}, $corepath.$corefile.${pathsepreg}, '^.*$' );

        # define hash value corresponing with the file
        $info{"TSLM copied $corefile"} = $srccore;
        debug 1, "TSLM copied $srccore\n";
    }
}

#
# get a list of files matching regexp in a directory
# sad multiplatform replacement for glob
#
sub get_direntries {
    my ($path, $regexp) = @_;
    return () if !-d "$path";

    opendir(DIR, "$path");
    my @direntries = readdir(DIR);
    closedir(DIR);

    return grep /$regexp/, @direntries;
}

#
# Copy files matching a regexp from given path to another path
#
sub copy_files {
    my ($srcpath, $dstpath,$regexp) = @_;
    my $copied = 0;
    debug 1, "copy files by regexp: $regexp\n";

    mkdir "$dstpath", S_IRWXU if (! -d "$dstpath");

    opendir(DIR, $srcpath) || printlog "can't read content of $srcpath.\n";
    my @direntries = readdir(DIR);
    closedir(DIR);

    foreach my $direntry (@direntries) {
        if (-f $srcpath.$direntry) {
            debug 1, "found file $direntry\n";

            if ($direntry =~ /$regexp/) {

                printlog 2,"Copy log file $direntry\n";
                copy( $srcpath.$direntry, "$dstpath$direntry")
                  or warning "Can't copy file $direntry.\n";
                $copied++;

                # define hash value corresponing with the file
                $info{"TSLM copied $direntry"} = "$srcpath$direntry";
                debug 1, "TSLM copied $srcpath$direntry\n";
            }
        }
    }
    return $copied;
}

#
# Copy TSLM log files from locations specified in conf files
#
sub collect_logsbyconffiles {
    printlog 1, "Copy log files from other locations\n";

    regexp_logsbyconffile( 'mm.xml',
                           '<TraceFile.*?>(.*?)<\/TraceFile>',
                           '^($file)(.\d+\.log)$' );

    regexp_logsbyconffile( 'genhdm.cfg',
                           '^\s*logfile\s*=\s*(\S.*?)$',
                           '^($file)(.\d+\.log)$' );

    regexp_logsbyconffile( 'dbuserexit.conf',
                           '^\s*ERMMDBLOGARCHIVE\s*=\s*(\S.*?);$',
                           '^(.*)$' );

    regexp_logsbyconffile( 'LibBalConf.xml',
                           '<mmtracepath\ name[\ =\"\']+(.*?)[\"\' ]+\/>.*<mmtracefile\ name[\ =\"\']+(.*?)[\"\' ]+\/>',
                           '^($file)(\.log|\.txt)$' );

    regexp_logsbyconffile( 'physicaldrive.conf',
                           '^\s*trace\s*=\s*(\S.*?)$',
                           '^($file)(\.log)$' );

   # get a list of all lm config files
    my @lmconf = grep s/TSLM copied (lm-.*\.xml)/\1/, keys %info;

    debug 1, "Found lm config files: @lmconf\n";

    foreach (@lmconf) {
        regexp_logsbyconffile( $_,
                               '<Tracing.*?>.*<TraceFile.*?>(.*?)<\/TraceFile>',
                               '^($file)(.\d+\.log)$' );
    }

}

#
# Parse config file with regexp for log files to copy.
# In the regexplog $file is replaced by the filename of
# the path and file combination matched with regexpparse.
#
sub regexp_logsbyconffile {
    my ($conffile, $regexpparse, $regexplog) = @_;
    debug 1, "Parse $conffile with \$regexpparse: $regexpparse   \$regexplog: $regexplog\n";

    # only continue if the conf file can be found
    return if !defined $info{"TSLM copied $conffile"};

    my $conf = readfile($info{"TSLM copied $conffile"});

    # parse file and search for $1 of regexp
    # return if regexp can't be found
    printlog 2, "Parse $info{\"TSLM copied $conffile\"} for log files\n";
    return if ! ($conf =~ /$regexpparse/ims);

    my $filepath = $1;
    if ($conffile eq 'LibBalConf.xml') {
        $filepath = $1.$2;
    }
    
    debug 1, "Found log file location: $filepath\n";

    # split path and filename of the filename found in the conf file
    my ($newpath,$newfile) = $filepath =~ /^(.*?)([^${pathsepreg}]*)$/;
    if ($newfile =~ /(.*)(\.log|\.txt)/) {
       $newfile = $1;
    }
    debug 1, "NEWPATH: $newpath; NEWFILE: $newfile\n";

    # use filename to complete regexp for filenames
    $regexplog =~ s/\$file/$newfile/;

    # now search in already copied files for the new logfile

    # get a list of all copied files
    my @copied = map $info{"TSLM copied $_"}, grep s/TSLM copied (.*)/\1/, keys %info;

    foreach (@copied) {
        # split path and filename of the already copied file
        my ($oldpath,$oldfile) = $_ =~ /^(.*?)([^$pathsepreg]*)$/;

        # apply regexplog to clean the file of probable sequence numbers
        my ($oldbase) = $oldfile =~ /$regexplog/;

        # we already copied the type of logfile so leave
        if ($newpath.$newfile eq $oldpath.$oldbase) {
            debug 1, "Log file already copied : $newpath$newfile\n";
            return;
        }
    }

    # file not yet copied. Copy to a seperate directory.
    debug 1, "Log file not copied: $newpath$newfile\n";
    copy_files( "$newpath", "$basename${pathsep}logs_scattered${pathsep}", $regexplog);
    debug 1, "Copying $newpath to $basename${pathsep}logs_scattered${pathsep}\n";
}

#
# read file into memory
# return file content in a skalar
#
sub readfile {
    my ($file) = @_;
    debug 2, "file: ".$file."\n";

    open(FILE,"<$file") || return "";
    my $oldfh = select FILE;
    local $/; # undef line seperator to read the file in one gulp
    select $oldfh;
    my $in = <FILE>;
    close(FILE);
    return $in;
}


#######################################################
#
# extract mm urls and ping mm
#

#
# extract mm urls from various config files
#
sub extract_urls {
    printlog 2, "Parse conf files for mm url\n";

    # extract url out of hdm config files
    my @urls = regexp_parseconffile( 'genhdm.cfg',
                                     '^\s*server\s*=\s*(\S.*?)$' );
    if (@urls) {
        $info{"TSLM url mm hdm"} = $urls[0];
        printsysinfo "MM url in genhdm.cfg: ".$info{"TSLM url mm hdm"}."\n";
    }

    # get a list of all lm config files
    my @lmconf = grep s/TSLM copied (lm-.*\.xml)/\1/, keys %info;
    debug 1, "Found lm config files: @lmconf\n";

    # extract url out of lm config files
    foreach (@lmconf) {
        my ($path,$file) = $_ =~ /^(.*?)([^$pathsepreg]*)$/;
        my @urls = regexp_parseconffile( $path.$file,
                                         '<MediaManager.*?>.*<Address.*?>(.*?)<\/Address>' );
        if (@urls) {
            $info{"TSLM url mm $file"} = $urls[0];
            printsysinfo "MM url in $file: ".$info{"TSLM url mm $file"}."\n";
        }
    }
}

#
# Parse config file with regexp and return matches
#
sub regexp_parseconffile {
    my ($conffile, $regexpparse) = @_;
    debug 1, "Parse $conffile with \$regexpparse: $regexpparse\n";

    # only continue if the conf file can be found
    return if !defined $info{"TSLM copied $conffile"};

    my $conf = readfile($info{"TSLM copied $conffile"});

    # parse file and search for $1 of regexp
    # return if regexp can't be found
    printlog 2, "Parse $info{\"TSLM copied $conffile\"} for regexp: $regexpparse\n";
    my @matches = $conf =~ /$regexpparse/imsg;
    debug 1, "Matches: @matches\n";
    return @matches;
}


sub ping_urls {
    # get a list of all urls
    my @cfgs = grep s/^TSLM url (.*)$/\1/, keys %info;
    printlog 1, "Ping TSLM urls\n";
    foreach my $cfg (@cfgs) {
        my ($prg, $file) = $cfg =~ /^(\S+) (.*)$/;
        printsysinfo "Ping $prg url \"".$info{"TSLM url $cfg"}."\" from config file $file:\n";
        open OUT,"ping \"".$info{"TSLM url $cfg"}."\" 2>&1 |";
        my $line = <OUT>;
        printsysinfo "$line";
        $line = <OUT>;
        printsysinfo "$line";
        $line = <OUT>;
        printsysinfo "$line";
        $line = <OUT>;
        printsysinfo "$line";
        close OUT;
    }
}


#######################################################
#
# Collect info via ermmtool
#

#
# Collect ermmtool info
#
sub collect_ermmtoolinfo {
	my $ermmtoolexec = DOS ? 'ermmtool.pl' : 'ermmtool';
    printlog 1, "Collect information via ermmtool ";
    if (! -f "$info{'TSLM path'}bin$pathsep$ermmtoolexec") {
        printlog 1, "\n";
        printlog 2, "Couldn't find ermmtool at $info{'TSLM path'}bin$pathsep$ermmtoolexec.\n";
        return;
    }
    my @commands = ( "lsai",       "lsapp",  "lscart", "lsct",      "lscg",
                     "lscga",      "lsdca",  "lsdg",   "lsdga",     "lsdm",
                     "lsdrive -l", "lslib",  "lslm",   "lsmess",
                     "lsml",       "lsreq",  "lsses",  "lssg",      "lsslot",
                     "lstask -l",  "lssys",  "lsvol",  "lsmonitor");
    open ERMMTOOL, ">$basename/ermmtool.log";
    foreach my $command (@commands) {
        animation;
        call_ermmtool("$info{'TSLM path'}bin${pathsep}ermmtool", $command);
    }
    close ERMMTOOL;
    printlog 1, " \n";
}

#
# Call ermmtool and record response
#
sub call_ermmtool {
    my ($ermmtool, $command) = @_;
    my $password_option = $ermmtool_password ne "" ? "-p $ermmtool_password" : ""; # FIX_FOR_PASSWORD
    my $cmdERMMtool = DOS ? "ermmtool $password_option $command" : "$ermmtool $password_option $command"; # FIX_FOR_PASSWORD
    debug 1,"$cmdERMMtool\n";
    open OUT,"$cmdERMMtool 2>&1 |";
    my @out = <OUT>;
    close OUT;
    chomp @out;
    print ERMMTOOL "Command: $command\n";
    foreach my $line (@out) {
        print ERMMTOOL "$line\n";
        animation;
    }
    print ERMMTOOL "\n";
}


#######################################################
#
# Collect system info via commands
#

#
# Collect system info
#
sub collect_sysinfo {
    printlog 1, "Collect system information\n";
    printsysinfo "System OS: $Config{'osname'}\n";
    printsysinfo "Perl version: $Config{'version'}\n";

    my @topics   = ( "Current user: ",
                     "Current system:\n",
                     "System limits: ",
                     "Disk usage:\n",
                     "Environment variables:\n",
                     "Directory:\n" );
    my @commands = ( "whoami",
                     "uname -a",
                     "ulimit -a",
                     "df",
                     "sh -c set",
                     "ls -lR \"$info{'TSLM path'}\"" );

    if (DOS) {
        @topics   = ( "Environment variables:\n", "Directory:\n" );
        @commands = ( "set", "dir /S \"$info{'TSLM path'}\"" );
    }
    for (my $i=0; $i<@commands; $i++) {
        call_sysinfo($topics[$i], $commands[$i]);
    }
}

#
# Call command and return output
#
sub call_cmd {
    my ($command) = @_;
    debug 1, "call_cmd: $command\n";
    open OUT,"$command 2>&1 |";
    my $oldfh = select OUT;
    local $/; # undef line seperator to read the file in one gulp
    select $oldfh;
    my $in = <OUT>;
    close OUT;
    return $in;
}

#
# Call system command and record output
#
sub call_sysinfo {
    my ($topic, $command) = @_;
    debug 1, "call_sysinfo: $topic   $command\n";
    my @out = split "\n",call_cmd($command);
    chomp @out;
    if (@out >= 1 && $out[0] =~ /is not recognized as an internal or external command,/) {
        chomp $topic;
        printsysinfo "$topic : not available\n";
    } else {
        printsysinfo "$topic";
        foreach my $line (@out) {
            printsysinfo "$line\n";
        }
    }
}


#######################################################
#
# Collect version information
#

#
# Collect version info in rpm systems
#
sub collect_versions {
    if (-f "/usr/bin/lslpp") {
        collect_versions_smit();
    }
    if (call_cmd('rpm --version') =~ /RPM version/) {
        collect_versions_rpm();
    }
    if (-f "/usr/sbin/swlist") {
        collect_versions_hpux();
    }
    if (REGISTRY && $info{'TSLM path'} =~ /^C:\\/) {
        collect_versions_win32();
    }
}

#
# Collect version info in rpm systems
#
sub collect_versions_rpm {
    printlog 1, "Collect versions of packages with rpm\n";
    my @ermmpackages = ( "tslm-mm",   "tslm-lm",     "tslm-system", "tslm-hdm",
                         "tslm-elm",  "tslm-libbal", "tslm-cli",    "tslm-3494", 
                         "tslm-pwgen" );
    my @otherpackages = ( "perl", "sg3_utils", "scsi" );
    my $command = "rpm -qi %s 2>&1";
    printsysinfo "TSLM rpm package versions: \n";
    foreach my $package (@ermmpackages) {
        call_packageinfo($command, $package);
    }
    printsysinfo "Support rpm package versions: \n";
    foreach my $package (@otherpackages) {
        call_packageinfo($command, $package);
    }
}

#
# Collect version info in aix systems
#
sub collect_versions_smit {
    printlog 1, "Collect versions of packages with lslpp\n";
    my @ermmpackages = ( "tslm.mm",   "tslm.lm",       "tslm.system",   "tslm.hdm",
                         "tslm.elm",  "tslm.libbal",   "tslm.cli",      "tslm.3494",
                         "tslm.pwgen" );
    my @otherpackages = ( "perl.rte" );
    my $command = "lslpp -L %s 2>&1 | head -3 | grep -v Description | grep -v -- -----";
    printsysinfo "TSLM smit package versions: \n";
    foreach my $package (@ermmpackages) {
        call_packageinfo($command, $package);
    }
    printsysinfo "Support smit package versions: \n";
    foreach my $package (@otherpackages) {
        call_packageinfo($command, $package);
    }
}

#
# Collect version info in HP-UX systems
#
sub collect_versions_hpux {
    printlog 1, "Collect versions of packages with swlist\n";
    printsysinfo "TSLM depot package versions: \n";
    my @depot = `swlist 2>&1 | grep -i tslm`; 
    chomp @depot;
    foreach my $line (@depot) {
        printsysinfo "$line\n";
    }
    printsysinfo "Support depot package versions: \n";
    my @perl_depot = `swlist perl 2>&1`;
    chomp @perl_depot;
    foreach my $line (@perl_depot) {
        printsysinfo "$line\n";
    }
}

#
# Call ermmtool and record response
#
sub call_packageinfo {
    my ($command, $package) = @_;
    $command =~ s/%s/$package/;
    debug 1,"$command\n";
    open OUT,"$command|";
    my @out = <OUT>;
    close OUT;
    chomp @out;
    foreach my $line (@out) {
        printsysinfo "$line\n";
    }
}

#
# Collect version info in windows systems
#
sub collect_versions_win32 {
    printlog 1, "Collect versions of packages from registry\n";
    printsysinfo "TSLM windows packages: \n";

    # find all keys
    my $ermmregkey = "Software\\Microsoft\\Installer\\Products";
    Win32API::Registry::RegOpenKeyEx( Win32API::Registry->HKEY_CURRENT_USER, "$ermmregkey", 0, Win32API::Registry->KEY_READ, my $keyhandle ) or return;
    my $uIndex = 0;
    my @keys = ();
    my $osName;
    while (Win32API::Registry::RegEnumKeyEx( $keyhandle, $uIndex++, $osName, my $iolName, [], my $osClass, my $iolClass, [] )) {
        push @keys, $osName;
        debug 1, "Found registry key: $osName\n";
    }

    Win32API::Registry::RegCloseKey( $keyhandle )
      or return;

  # HDM, ELM
  #my @keys = ("1B05E2A451415724B87257D82F1BAE66", "5F8DF2D65114B644BA3BE339D107AACB");

  foreach my $key (@keys) {
    my $ermmregkey = "Software\\Microsoft\\Installer\\Products\\$key";
    Win32API::Registry::RegOpenKeyEx( Win32API::Registry->HKEY_CURRENT_USER, "$ermmregkey", 0, Win32API::Registry->KEY_READ, my $keyhandle )
      or next;
    if (Win32API::Registry::RegQueryValueEx( $keyhandle, "ProductName", [], my $type, my $data, [] )) {
        if ($data =~ /^TSLM /) {
            printsysinfo "$data\n";
        }
    }
    Win32API::Registry::RegCloseKey( $keyhandle )
      or next;
  }
}


#######################################################
#
# Test DB2 Database
#

#
# test connection to db2
#
sub test_db2 {
    printlog 1, "Perform DB2 access test\n";
    my $db2path= "db2";

    # guess db2 location in case it isn't in the path
    # sad replacement for glob('/opt/IBM/db2/*/bin/db2')
    if (-d "/opt/IBM/db2/") {
        opendir(DIR, "/opt/IBM/db2/");
        my @direntries = readdir(DIR);
        closedir(DIR);

        foreach my $direntry (@direntries) {
            if (-f "/opt/IBM/db2/$direntry/bin/db2") {
                $db2path = "/opt/IBM/db2/$direntry/bin/db2";
                debug 1,"db2path: $db2path\n";
            }
        }
    }

    # connect to db2 ermm db
    open OUT, "$db2path connect to ermm 2>&1|";
    my @out = <OUT>;
    close OUT;
    open OUT, "$db2path connect reset 2>&1|";
    close OUT;

    chomp @out;

    printsysinfo "db2 connect:\n";

    foreach my $line (@out) {
        printsysinfo "$line\n";
    }
}

#######################################################
#
# Build Package
#

#
# Pack info directory to tar package
#
sub logpackage_tar {
    printlog 2, "Tar log package directory $basename/\n";
    open OUT,"tar cf \"$basename.tar\" \"$basename\" 2>&1 |";
    my @out = <OUT>;
    close OUT;
    chomp @out;
    if (! -f "$info{'result file'}.tar") {
        warning "tar failed. $info{'result file'}.tar does not exist.\n";
        warning "tar out: @out\n";
        return;
    }
    $info{'result file'} .= ".tar";
}

#
# Compress tar file
#
sub logpackage_zip {
    debug 1, "Compress log package $info{'result file'}\n";
    open OUT,"gzip -f \"$info{'result file'}\" 2>&1 |";
    my @out = <OUT>;
    close OUT;
    chomp @out;
    if (! -f "$info{'result file'}.gz") {
        printlog 2, "gzip failed. Probably not installed.\n";
        debug 1, "gzip out:\n".join("\n",@out)."\n";
        return;
    }
    $info{'result file'} .= ".gz";
}

#
# use perl Archive:Tar Module to pack the log files
#
sub logpackage_perltar {
    my $tar = Archive::Tar->new;
    printlog 2, "Tar log package directory/\n";
    printlog 2, "Use perl Archive::Tar/\n";
    tar_addpath($tar, "$basename/");
    $tar->write( "$basename.tar" );
    $info{'result file'} .= ".tar";
}

#
# add files in path to Archive:Tar package
#
sub tar_addpath {
    my ($tar, $path)= @_;

    opendir(DIR, $path) || die "can't open $path\n";
    my @direntries = readdir(DIR);
    closedir(DIR);
    debug 1, "Add path to tar: $path\n";

    foreach my $direntry (@direntries) {
        next if $direntry eq ".";
        next if $direntry eq "..";
        if (-d $path.$direntry) {
            tar_addpath($tar, $path.$direntry."/");
        }
        if (-f $path.$direntry) {
            debug 1, "Add file to tar: $path.$direntry\n";
            $tar->add_files( $path.$direntry );
        }
    }
}


#######################################################
#
# Main
#

printlog 1, "Build log package\n";

# Create basic fileame
my ($sec, $min, $hour, $mday, $mon, $year, $wday, $jday, $isdst) = localtime;
# adjust month offset
$mon++;
$basename .= sprintf "-%d%2.2d%2.2d-%2.2d%2.2d",$year+1900,$mon,$mday,$hour,$min;
$info{'basename'}= $basename;
$basename = "$options{'temp'}$basename";

# check if we have a valid temporary directory at all.
debug 1, "Check for a temporary directory: $options{'temp'}\n";
-d $options{'temp'} || die "Error: Can't access temporary directory $options{'temp'}";

# just allow the current user to see the files created from now on
my $oldumask= umask(umask() | oct("077"));
debug 1, "Preserve umask: $oldumask   New umask: ".umask()."\n";

debug 1, "Create log package directory: $basename/\n";
mkdir $basename, S_IRWXU || die "Error: Can't create directory $basename$pathsep\n";
printlog 1, "Create $basename$pathsep\n";

# Start by creating a log file and check if this file can be accessed
open LOG, ">$basename/ermmCollectLogs.log" || die "Can't write logs to directory $basename";
if (!-f "$basename/ermmCollectLogs.log") {
    printlog 1, "Can't write logs to directory $basename.\n";
    printlog 1, "Please check that the current user has access to the\n";
    printlog 1, "temporary directory and that $basename doesn't exist.\n";
    exit;
}
printlog 2, "Activity log for ermmCollectLogs\n";

printlog 2, "Open system information file systemInfo.txt\n";
open SYSINFO, ">$basename/systemInfo.txt";
printsysinfo "Collected information about the current system\n";

# Perform tests and collect logs and information
collect_ermmlocation;

collect_conffiles;

collect_logfiles;

collect_logsbyconffiles;

collect_corefiles;

collect_ermmtoolinfo;

collect_versions;

collect_sysinfo;

test_db2;

extract_urls;

ping_urls;

close SYSINFO;
close LOG;

$info{'result file'} = $basename;

# tar the log package
printlog 1, "Pack log package\n";

if (DOS && TAR) { # FIX_FOR_MAKING_TAR_FILE
    logpackage_perltar;
} else {
    logpackage_tar;
}

# zip it
logpackage_zip;

# copy result to location specified with the -o command line parameter
if (($info{'result file'} ne $options{'out'}) && ($options{'out'} ne "")) {
    if (copy( $info{'result file'}, $options{'out'})) {
        $info{'result file'} = $options{'out'};
        debug 1, "Copy to final location\n";
    } else {
        warning "Can't copy log package $info{'result file'} to $options{'out'}.\n";
        warning "$!\n";
    }
}

# reset file permission
debug 1, "Restore umask: $oldumask\n";
umask( $oldumask);

printlog 1, "\n";

if (-f $info{'result file'}) {
    printlog 1, "Successfully created log package. Please email IBM support\n";
    printlog 1, "the file: $info{'result file'}\n";
} elsif (-d $info{'result file'}) {
    printlog 1, "Check if the directory $info{'result file'}\n";
    printlog 1, "has been created on your system. Pack and email\n";
    printlog 1, "the directory to IBM support.\n";
} else {
    printlog 1, "Failed to create log package.\n";
}

# on windows wait for a keypress since the file might have been started from the gui.
if (DOS && READKEY) {
    printlog 1, "Please press any key to quit.\n";

    my $key;
    Term::ReadKey::ReadMode(4); # Turn off control keys
    while (! defined ($key = Term::ReadKey::ReadKey(-1))) {
        sleep 1; # Don't use up all resources while waiting for keypress

    }
    Term::ReadKey::ReadMode(0); # Reset tty mode before exiting
}

1;
