#!/usr/bin/perl
use strict;
#-----------------------------------------------------------------------
# Licensed Materials - Property of IBM
#
# IBM Tape System Library Manager 
#
# (C) Copyright IBM Corp. 2004, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use,  duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#-----------------------------------------------------------------------
#
#
# @version 1.8 - 15/02/04 23:06:21
#
#
#
# @brief audit3494  - script that monitors import/export activity and
#                     emulates 3494 behaviour
#
#-----------------------------------------------------------------------
# configuration variables
# $DEBUG - debug	>0: display debug messages
# $targetLibsMode ["ALL"|"PART"]  - scope of libraries
# $sleepTime - seconds to sleep for monitor interval
# $aiName - "Audit3494" - name of application instance
# $auditFrom - begin address of I/O slot
# $auditTo - end address of I/O slot
# where, audit operation to recognize new cartridge will be
# performed between $auditFrom and $auditTo
# $numberOfEmptySlot - specify the number of empty slot as destination for moving cartridge from I/O to internal
# $fullAuditIntervalTime - specify an interval time[sec] for full audit. 
# $fullAuditIntervalTimeBtLib - specify a wait time during full audit operation for each library

my $targetLibsMode="ALL";
my $targetLibs = "";    # not used in case of $targetLibsMode="ALL"
my $sleepTime=120;      # seconds to sleep for monitor interval
my $aiName="Audit3494";
my $auditFrom="769";
my $auditTo="1024";
my $numberOfEmptySlot="1..255";
my $fullAuditIntervalTime=60*60*24; #3600sec=24hour
my $fullAuditIntervalTimeBtLib=60;  #60sec

# Turn on some print statements for debugging only..
my $DEBUG=0;

# signal handler for interval time of full audit
$SIG{ALRM}=\&fullAuditIntervalTimeHandler;
alarm $fullAuditIntervalTime;
my $fullAuditFlag=0;

#============ don't change anything below this line ====================
#=========== unless you really know what you're doing ==================
my $cmd = "/opt/IBM/ermm/bin/ermmtool";
my @libNames; #array of library names  "libname1", "libname2", ... 
my @carts;    #array of pcl names "cart1", "cart2", ...
my @libcarts; #array of references hash ("lib" => "$libName", "cart" => "$cart" )
my @libEmptyLsSlots; #array of references hash ("lib" => "$libName", "slot" => "$slot" )

#
# get target libraries
#
getLibNames();
#
# ensure Audit3494 AI exist and has admin permissions
#
my $res = `$cmd addai $aiName ERMMSystem`;
$res = `$cmd chai -adm -app ERMMSystem $aiName`;

#
# ensure Audit3494 AI is being used for all command
# so that we can trace its behaviour in the log
#
$cmd .= " -i $aiName";

my $rv;
my $cart;
my $intCount = 0;

while (1) {
    auditLibraries();
    # get all newly detected cartridges (= in NoAccess group) into @libcarts
    # and make them available in the inject category
    #
    # where, @libcarts is array of reference hash ("lib" => "$libName", "cart" => "$cart" )
    #
    getCartridges("NoAccess",@libNames);

    # get empty internal slots for each library that's used for destination of cartridge inject
    # where, @libEmptyLsSlots is array of references hash ("lib" => "$libName", "slot" => "$slot" )
    getEmptyLsSlots(@libNames);
    print "Import will start shortly\n" if $DEBUG>0;

    foreach my $libcart (@libcarts) {
	my $lib = $$libcart{lib}; # pick library name from hash reference
	$cart= $$libcart{cart};   # pick cartridge name from hash reference
	my $vol = substr($cart, 0, 6);  # generate volume name from cartridge name

	print "Detected $cart in NoAccess category\n" if $DEBUG>0 ;

	# move cartridge into inject category (CartridgeGroup 0xff00)
	$rv = `$cmd chcart $cart -group 0xff00 `;
	if ($rv =~ /\w{3}\d{4}\w:/) {
	    $rv = `$cmd addlog -e \"$aiName: error while moving cartridge $cart to cartridgegroup 0xff00\"`;
	    print "- Fail to moved $cart into cartridgegroup 0xff00 slot\n" if $DEBUG>0 ;
	    # put back this cart into NoAccess group
	    $rv = `$cmd chcart $cart -group NoAccess `;
	} else {
	    print "- Moved $cart into cartridgegroup 0xff00 slot\n" if $DEBUG>0 ;
	    # allocate volume on cartridge
	    $rv = `$cmd -c 3494LM -i 3494LM-1 lsvol $vol`;
	    # If not allocated, above command fails, and it will try to allocate for 3494LM, if not pass down into ELSE...
	    if ($rv =~ /\w{3}\d{4}\w:/) { $rv = `$cmd -c 3494LM -i 3494LM-1 alloc -pcl $cart $vol`; }
	    if ($rv =~ /\w{3}\d{4}\w:/) {
		$rv = `$cmd addlog -e \"$aiName: error while allocating volume on cartridge $cart\"`;
		print "- Could not allocate volume $vol to 3494 Emualtor\n" if $DEBUG>0 ;
		# deallocate this volume if it is allocated for some reason
		$rv=`$cmd dealloc -app 3494LM $vol`;
		# put back this cart into NoAccess group
		$rv = `$cmd chcart $cart -group NoAccess `;
	    } else {
		print "- Allocated volume $vol to 3494 Emualtor\n" if $DEBUG>0 ;
		# move volume into a storage slot
		$rv = moveCartridgeToStorageSlot($libcart);
		if ($rv == 0) {
		    print "- Moved $cart into internal slot\n" if $DEBUG>0 ;
		    $rv = `$cmd addlog -i \"$aiName: cartridge $cart imported successfully\"`;
		} else {
		    print "- Failed to move $cart into internal slot\n" if $DEBUG>0 ;
		    # move cart back into I/O is not necessary 
		    # deallocate this volume if it is allocated for some reason
		    $rv=`$cmd dealloc -app 3494LM $vol`;
		    # put back this cart into NoAccess group
		    $rv = `$cmd chcart $cart -group NoAccess `;
		}
	    }
	}
    }
    
    #
    # move  all cartridges in eject category into an I/O slot
    #
    getCartridges("0xff10",@libNames);
    
    # in order to move cartridge to I/O slot by moveCartridgeToIOSlot()
    # library information is not required. So, the only cartridge array is extracted.
    @carts=map { $$_{cart} } @libcarts;

    print "There are ",scalar @carts," cartridges in 0xff10 category\n" if $DEBUG>0;

    print "Export will start shortly\n" if $DEBUG>0;
    
    foreach $cart (@carts) {
	$rv = moveCartridgeToIOSlot($cart);
	if ($rv == 0) {
	    $rv = `$cmd addlog -i \"$aiName: cartridge $cart exported successfully\"`;
	    print "- Move $cart into I/O slot\n" if $DEBUG>0 ;
	} else {
	    print "- Failed to move $cart into I/O slot\n" if $DEBUG>0 ;
	}
    }

    #
    # get all cartridges physically removed from the library and remove them from repository
    # when cartridges are removed from the library, they will be automatically taken out of TSLM/ermmm DB..
    #
    getEjectedCartridges();
    print "There are ",scalar @carts," cartridges ejected according to the DB2 entries\n" if $DEBUG>0;
    
    foreach (@carts) {
	my $vol;
	s/\s//g;
	$cart = $_;
	$vol = substr($cart, 0, 6);

	# deallocate volume
	$rv = `$cmd dealloc -app 3494LM $vol`;
	if ($rv =~ /\w{3}\d{4}\w:/) {
	    $rv = `$cmd addlog -e \"$aiName: error while deallocating cartridge $cart\"`;
	    print "- Failed to deallocate $vol from 3494 Emulator\n" if $DEBUG>0 ;
	} else {
	    print "- Deallocated $vol from 3494 Emulator\n" if $DEBUG>0 ;
	    # remove volume from database
	    $rv = `$cmd rmcart $cart`;
	    if ($rv =~ /\w{3}\d{4}\w:/) {
		$rv = `$cmd addlog -e \"$aiName: error while removing cartridge $cart\"`;
		print "- Failed to remove $cart from TSLM/ermm database\n" if $DEBUG>0 ;
	    } else {
		$rv = `$cmd addlog -i \"$aiName: cartridge $cart removed from repository\"`;
		print "- Remove $cart from TSLM/ermm database\n" if $DEBUG>0 ;
	    }
	}
    }

    print "Sleeping for $sleepTime seconds before auditing again. \n" if $DEBUG>0;
    # sleep before starting all over again
    sleep($sleepTime);
}

#
# get cartridges from a specific CartridgeGroup provided as parameter
# into @libcart array
#    
sub getCartridges
{
    my ($group, @_libNames)=@_;
    my $result, $rv;

    @libcarts = ();
    foreach my $libName ( @_libNames ){
    	$result = `$cmd lscart -lib $libName -s -group $group -hdr off`;
    	chomp($result);

	# Check for error message number
    	if ($result =~ /^\w{3}\d{4}\w:/) {
	    if (substr($result, 0, 8) eq "EMM7009I") {
		# no matching cartridges found (ignore this error)
		next;
	    } else {
		$rv = `$cmd addlog -e \"error while executing lscart command with -lib=$libName -group=$group\"`;
		next;
	    }
	} else {
	    my @_carts=split(/\n/,$result);
	    print "There are ",scalar @_carts," cartridges in NoAccess category in the logical library $libName\n" if $DEBUG>0; 
	    push( @libcarts, map +{ ("lib" => "$libName", "cart" => "$_" ) }, @_carts );
	}
    }
    return 1;
}

#
# get empty slots for each library
# into @libEmptyLsSlots array
#    
sub getEmptyLsSlots
{
    my (@_libNames)=@_;
    my $result, $rv;

    @libEmptyLsSlots = ();
    foreach my $libName ( @_libNames ){
	$result=`$cmd lsslot -lib $libName -empty -hdr off -sg internal -number $numberOfEmptySlot | cut -f1 -d\' \'`;
        print "empty slots ($libName):\n$result" if $DEBUG>0;
    	chomp($result);

	# Check for error message number
    	if ($result =~ /^\w{3}\d{4}\w:/) {
	    if (substr($result, 0, 8) eq "EMM7009I") {
		# no matching cartridges found (ignore this error)
		next;
	    } else {
		$rv = `$cmd addlog -e \"error while executing lsslot command with -lib=$libName \"`;
		next;
	    }
	} else {
	    my @_slots=split(/\n/,$result);
	    print "There are ",scalar @_slots," empty slots in the logical library $libName\n" if $DEBUG>0; 
	    push( @libEmptyLsSlots, map +{ ("lib" => "$libName", "slot" => "$_" ) }, @_slots );
	}
    }
    return 1;
}

sub getEmptySlot
{
    my $libName = shift;
    my @_libSlots = grep { $$_{lib} eq "$libName" } @libEmptyLsSlots;
    my @libSlots = sort { int(rand(3)) - 1 } @_libSlots;
    my $libSlot;
    foreach my $_libSlot (@libSlots){
	if (!isHomeSlots($$_libSlot{slot})){
	    $libSlot=$_libSlot;
	    last;
	}
    }
    if (!defined($libSlot)){
	return undef;
    }
    my @_libEmptyLsSlots = grep { $$_{lib} ne $$libSlot{lib} || $$_{slot} ne $$libSlot{slot} } @libEmptyLsSlots;
    my @libEmptyLsSlots = @_libEmptyLsSlots;
    return $$libSlot{slot};
}

#
# utilize DB2 command to get list of cartridges that have been 
# physically removed
#
sub getEjectedCartridges
{
    my $count;

    @carts = ();
    my $result = `db2 connect to ermm >/dev/null && db2 -x select cartridgepcl from m_cartridge where libraryid is null && db2 -x disconnect current >/dev/null`;

    @carts = split(/\n/,$result);

    return 0;
}

#
# check if destination is not home slot of mounted cartridges
# input parameter
# $1: slotname
# return value
# 0: not homeslot
# 1: homeslot
#
sub isHomeSlots
{
    my $slot = shift;
    my @homeLsSlots=`$cmd lsmp -l -hdr off`;
    print @homeLsSlots if $DEBUG>0;
    # lsmp shows following info:
    # <id> <lib> <drive> <pcl> <app> <slot> <lastdate>
    #  0    1     2       3     4     5      6
    # we need only <slot> field 
    #ermmtool> lsmp -l
    #ID                                   Library Drive      PCL      Application Slot TimeMounted
    #===============================================================================================
    #9a46d2d8-3603-11e3-9b96-00215e6d04d0 3584-1  3584-1-259 J1H313JA ERMMSystem  1026 2013-10-16T10:38:05
    my @slots = map {split / /;
		     @_[5];
		 }@homeLsSlots;
    print "targetslot=$slot\n" if $DEBUG>0;
    print "homeslots=@slots\n" if $DEBUG>0;
    my $result = grep {$_ eq "$slot"} @slots;
    print "isHomeSlot()=$result\n" if $DEBUG>0;
    if ($result eq "1" ){
	return 1;
    }else{
	return 0;
    }
}

#
# move cartridge (pcl and library name provided as parameter) into 
# a storage slot inside the library if it currently
# resides in an I/O slot
#
sub moveCartridgeToStorageSlot
{
    my $libcart = shift;
    my $lib = $$libcart{lib};
    my $cart= $$libcart{cart};
    my $rv;
    my $inio = 0;

    # get slot the cartridge resides in
    $rv = `$cmd lsslot -match \"streq(Cartridge.\\\"CartridgePCL\\\" \\\"$cart\\\")\" -hdr=off`;

    # check if cartridges resides in I/O slot
    if ($rv =~ /.*\s+I\/O\s+.*/) {
        my $destSlot = getEmptySlot($lib);

	while (defined ($destSlot)){
	    
	    # check if destination is empty and pick another candidate if it's needed 
            print "seq1:$cmd lsslot -lib $lib -empty $destSlot -hdr=off -number $numberOfEmptySlot\n" if $DEBUG>0;
	    $rv = `$cmd lsslot -lib $lib -empty $destSlot -hdr=off -number $numberOfEmptySlot`;
	    while ($rv =~ /EMM7009I:/) { #find another candidate in case of occupied destination
		$destSlot = getEmptySlot($lib);
		if (!defined($destSlot)) {
		    print "-- Moving $cart into internal slot has failed.\n" if $DEBUG>0 ;
		    $rv = `$cmd addlog -e \"$aiName: error while moving cartridge $cart to storage slot\"`;
		    return 1;
		}
                print "seq2:$cmd lsslot -lib $lib -empty $destSlot -hdr=off -number $numberOfEmptySlot\n" if $DEBUG>0;
		$rv = `$cmd lsslot -lib $lib -empty $destSlot -hdr=off -number $numberOfEmptySlot`;
	    }

	    # Move to any internal slot and let the library decide on where to 
            print "seq3:$cmd move -pcl $cart -toslot $destSlot\n" if $DEBUG>0;
	    $rv = `$cmd move -pcl $cart -toslot $destSlot`;
            if ($rv =~ /EMM2S01E:/) { #retry action in case of destination full
		$destSlot = getEmptySlot($lib);
            } elsif ($rv =~ /EMM2909E:/) { #retry action in case of destination full
		$destSlot = getEmptySlot($lib);
	    } elsif ($rv =~ /\w{3}\d{4}\w:/) {
		print "-- Moving $cart into internal slot has failed.\n" if $DEBUG>0 ;
		$rv = `$cmd addlog -e \"$aiName: error while moving cartridge $cart to storage slot\"`;
		return 1;
	    } else {
		chomp($rv);
		print "-- Moving $cart into internal slot is completed succesfully.\n" if $DEBUG>0 ;
		return 0;
	    }
	}
	print "-- Moving $cart into internal slot has failed.\n" if $DEBUG>0 ;
	$rv = `$cmd addlog -e \"$aiName: error while moving cartridge $cart to storage slot\"`;
	return 1;
    } elsif ($rv =~ /.*\s+internal\s+.*/) {
	print "-- $cart is already in internal slot, skipping the move.\n" if $DEBUG>0;
	return 0;
    } else {
	return 1;
    }
}

#
# move cartridge (pcl and Library name provided as parameter) into 
# an I/O slot of the library if it currently
# resides inside the library in an ordinary storage slot
#
sub moveCartridgeToIOSlot
{
    my $cart = shift;
    my $rv;
    my $inio = 0;

    # get slot the cartridge resides in
    $rv = `$cmd lsslot -match \"streq(Cartridge.\\\"CartridgePCL\\\" \\\"$cart\\\")\" -hdr=off`;

    # check if cartridge resides in an internal storage slot
    if ($rv =~ /.*\s+internal\s+.*/) {
	$rv = `$cmd move -pcl $cart -tosg \"I/O\" `;
	if ($rv =~ /\w{3}\d{4}\w:/) {
	    print "-- Moving $cart into internal slot has failed.\n" if $DEBUG>0 ;
	    $rv = `$cmd addlog -e \"$aiName: error while moving cartridge $cart to I/O slot\"`;
	    return 1;
	} else { 
	    print "-- Moving $cart into IO slot is completed succesfully.\n" if $DEBUG>0 ;
	    return 0;
	}
    } elsif ($rv =~ /.*\s+I\/O\s+.*/) {
        print "-- $cart is already in I/O slot, skipping the move.\n" if $DEBUG>0;
        return 0;
    } else {
        return 1;
    }
}

#
# get target libraries
#
sub getLibNames
{
    if ($targetLibsMode eq "ALL") {
	@libNames = `$cmd lslib --hdr off -s 2>/dev/null`;
	until ( @libNames ) {
	    print "Can not get the library names, sleeping for $sleepTime seconds before checking again\n" if $DEBUG>0;
	    sleep($sleepTime);
	    @libNames = `$cmd lslib --hdr off -s 2>/dev/null`;
	    print "Checking for available libraries\n" if $DEBUG>0;
	}
    }else{
	@libNames = split(/,/,$targetLibs);
    }
    # Remove newlines
    chomp(@libNames);


    print "Found these logical libraries:  @libNames\n" if $DEBUG>0;
}

#
# audit libraries
#
sub auditLibraries
{
    my $BtLibFlag=0;
    foreach my $libName ( @libNames ) {
	print "Auditing the logical library $libName\n" if $DEBUG>0; 
	#
	# audit library
	#
	if ($fullAuditFlag==1){
	    if ($BtLibFlag!=0){
		$BtLibFlag=1;
		sleep $fullAuditIntervalTimeBtLib;
	    }
	    print "Perform full audit for $libName\n" if $DEBUG>0;
	    $rv = `$cmd audit $libName`;
	}else{
	    $rv = `$cmd audit -from $auditFrom -to $auditTo $libName`;
	}
	if ($rv =~ /\w{3}\d{4}\w:/) {
	    $rv = `$cmd addlog -e \"$aiName: error while auditing library $libName\"`;
	}
    }
    if ($fullAuditFlag==1){
	$fullAuditFlag=0;
	alarm $fullAuditIntervalTime;
    }
}

sub fullAuditIntervalTimeHandler{
    $fullAuditFlag=1;
    alarm 0;
}
